#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Mar 18 20:14:10 2022

@author: marina
"""
import numpy as np
from skimage.util import view_as_blocks 
from scipy.stats import binom


def Correlation(B1, B2):
    aux_B1 = B1.reshape(-1) - np.mean(B1)
    aux_B2 = B2.reshape(-1) - np.mean(B2)
    corr = np.dot(aux_B1, aux_B2) / (np.linalg.norm(aux_B1) * 
                                     np.linalg.norm(aux_B2))
    
    return corr

def CrossCorr(array1, array2):
    """
    Computes 2D cross-correlation of two 2D arrays.
    
    """ 
    array1 = array1.astype(np.double)
    array2 = array2.astype(np.double)
    array1 = array1 - array1.mean()
    array2 = array2 - array2.mean()

    normalizator = np.sqrt(np.sum(np.power(array1,2))*np.sum(np.power(array2,2)))
    tilted_array2 = np.fliplr(array2);     del array2
    tilted_array2 = np.flipud(tilted_array2)
    TA = np.fft.fft2(tilted_array2);           del tilted_array2
    FA = np.fft.fft2(array1);                  del array1
    AC = np.multiply(FA, TA);                  del FA, TA

    if normalizator==0:
        ret = None
    else:
        ret = np.real(np.fft.ifft2(AC))/normalizator
    return ret

def _RN(C, size):
    '''
    sets to 0 a 2-D square neighborhood of size (size x size) around [-1,-1] from matrix C
    
    '''
    if size == int(size/2):
        print('size must be odd')
    else:
        rad = int(size/2) + 1
        aux = np.roll(C, [rad,rad], axis = [0,1])
        aux[0:size,0:size] = 0
        return aux

def SPCE(C, squaresize=11):
    corr = C[-1,-1]
    C_RN = _RN(C,squaresize)
    PCE_energy = np.sum(C_RN*C_RN)/(C.shape[0]*C.shape[1]-squaresize**2)
    spce = np.sign(corr)*corr**2/PCE_energy
    return spce

def PCE(C, squaresize=11):
    corr = C[-1,-1]
    C_RN = _RN(C,squaresize)
    PCE_energy = np.sum(C_RN*C_RN)/(C.shape[0]*C.shape[1]-squaresize**2)
    pce = corr**2/PCE_energy
    return pce

def CompareSignedPCE(img_noise1, img_noise2):
    C = CrossCorr(img_noise1, img_noise2)
    spce = SPCE(C)
    return spce

def ComparePCE(img_noise1, img_noise2):
    C = CrossCorr(img_noise1, img_noise2)
    P = PCE(C)    
    return P

def CompareNFA(img_noise1, img_noise2, W):
    x = int(img_noise1.shape[0]/W)
    y = int(img_noise1.shape[1]/W)
    img_noise1 = img_noise1[0: x*W, 0:y*W]
    img_noise2 = img_noise2[0: x*W, 0:y*W]
    
    # extract WxW blocks from fingerprint and from noise image
    img_noise_blocks = view_as_blocks(img_noise1, (W,W)).reshape(-1, W, W)
    fingerprint_blocks = view_as_blocks(img_noise2, (W,W)).reshape(-1, W, W)
    # define variable where to store votes, one vote per block
    num_blocks = img_noise_blocks.shape[0]
    votes = np.zeros(num_blocks)
    # compute votes
    for i in range(num_blocks):
        #print(f'Bloque {i} de un total de {num_blocks}')
        # get noise image block
        img_noise_block = img_noise_blocks [i,:,:]
        # variables to be updated
        max_corr = -10
        pos_max_corr = -1
        for j in range(num_blocks):
            # get fingerprint block
            fp_block = fingerprint_blocks [j,:,:]
            # compute correlation between noise block and fingerprint block
            blocks_corr = Correlation( img_noise_block, fp_block ) 
            if blocks_corr > max_corr:
                max_corr = blocks_corr
                pos_max_corr = j
        # compute block i's vote !
        if pos_max_corr == i:
            votes[i] = 1
    total_votes = np.sum(votes)
    NFA = 1 - binom.cdf( total_votes - 1, num_blocks, 1 / num_blocks )
    return NFA